@@ -1,18 +1,29 @@
-#===============================================================================
-# Copyright 2010 Matt Chaput
-# 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-# 
-#    http://www.apache.org/licenses/LICENSE-2.0
-# 
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-#===============================================================================
+# Copyright 2010 Matt Chaput. All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
+#
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY MATT CHAPUT ``AS IS'' AND ANY EXPRESS OR
+# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
+# EVENT SHALL MATT CHAPUT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
+# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+#
+# The views and conclusions contained in the software and documentation are
+# those of the authors and should not be interpreted as representing official
+# policies, either expressed or implied, of Matt Chaput.
 
 from __future__ import division
 import os.path
@@ -37,7 +48,7 @@ except ImportError:
 
 try:
     from persistent import Persistent
-    
+
     class ZDoc(Persistent):
         def __init__(self, d):
             self.__dict__.update(d)
@@ -50,19 +61,19 @@ class Module(object):
         self.bench = bench
         self.options = options
         self.args = args
-    
+
     def __repr__(self):
         return self.__class__.__name__
-    
+
     def indexer(self, **kwargs):
         pass
-    
+
     def index_document(self, d):
         raise NotImplementedError
-    
+
     def finish(self, **kwargs):
         pass
-    
+
     def _process_result(self, d):
         attrname = "process_result_%s" % self.options.lib
         if hasattr(self.bench.spec, attrname):
@@ -72,19 +83,19 @@ class Module(object):
         else:
             self._process_result = lambda x: x
             return d
-    
+
     def searcher(self):
         pass
-    
+
     def query(self):
         raise NotImplementedError
-    
+
     def find(self, q):
         raise NotImplementedError
-    
+
     def findterms(self, terms):
         raise NotImplementedError
-    
+
     def results(self, r):
         for hit in r:
             yield self._process_result(hit)
@@ -93,17 +104,17 @@ class Module(object):
 class Spec(object):
     headline_field = "title"
     main_field = "body"
-    
+
     def __init__(self, options, args):
         self.options = options
         self.args = args
-        
+
     def documents(self):
         raise NotImplementedError
-    
+
     def setup(self):
         pass
-    
+
     def print_results(self, ls):
         showbody = self.options.showbody
         snippets = self.options.snippets
@@ -111,40 +122,41 @@ class Spec(object):
         for i, hit in enumerate(ls):
             if i >= limit:
                 break
-            
-            print "%d. %s" % (i + 1, hit.get(self.headline_field))
+
+            print("%d. %s" % (i + 1, hit.get(self.headline_field)))
             if snippets:
-                print self.show_snippet(hit)
+                print(self.show_snippet(hit))
             if showbody:
-                print hit.get(self.main_field)
+                print(hit.get(self.main_field))
 
 
 class WhooshModule(Module):
     def indexer(self, create=True):
         schema = self.bench.spec.whoosh_schema()
-        path = os.path.join(self.options.dir, "%s_whoosh" % self.options.indexname)
-        
+        path = os.path.join(self.options.dir, "%s_whoosh"
+                            % self.options.indexname)
+
         if not os.path.exists(path):
             os.mkdir(path)
         if create:
             ix = index.create_in(path, schema)
         else:
             ix = index.open_dir(path)
-        
+
         poolclass = None
         if self.options.pool:
             poolclass = find_object(self.options.pool)
-        
+
         kwargs = dict(limitmb=int(self.options.limitmb), poolclass=poolclass,
                       dir=self.options.tempdir, procs=int(self.options.procs),
                       batchsize=int(self.options.batch))
-        
+
         if self.options.expw:
             from whoosh.filedb.multiproc import MultiSegmentWriter
             self.writer = MultiSegmentWriter(ix, **kwargs)
         else:
             self.writer = ix.writer(**kwargs)
-        
+
         self._procdoc = None
         if hasattr(self.bench.spec, "process_document_whoosh"):
             self._procdoc = self.bench.spec.process_document_whoosh
@@ -157,20 +169,22 @@ class WhooshModule(Module):
 
     def finish(self, merge=True, optimize=False):
         self.writer.commit(merge=merge, optimize=optimize)
-        
+
     def searcher(self):
-        path = os.path.join(self.options.dir, "%s_whoosh" % self.options.indexname)
+        path = os.path.join(self.options.dir, "%s_whoosh"
+                            % self.options.indexname)
         ix = index.open_dir(path)
         self.srch = ix.searcher()
-        self.parser = qparser.QueryParser(self.bench.spec.main_field, schema=ix.schema)
-        
+        self.parser = qparser.QueryParser(self.bench.spec.main_field,
+                                          schema=ix.schema)
+
     def query(self):
         qstring = " ".join(self.args).decode("utf8")
         return self.parser.parse(qstring)
-    
+
     def find(self, q):
         return self.srch.search(q, limit=int(self.options.limit))
-    
+
     def findterms(self, terms):
         limit = int(self.options.limit)
         s = self.srch
@@ -178,14 +192,15 @@ class WhooshModule(Module):
         for term in terms:
             q.text = term
             yield s.search(q, limit=limit)
-            
+
 
 class XappyModule(Module):
     def indexer(self, **kwargs):
-        path = os.path.join(self.options.dir, "%s_xappy" % self.options.indexname)
+        path = os.path.join(self.options.dir, "%s_xappy"
+                            % self.options.indexname)
         conn = self.bench.spec.xappy_connection(path)
         return conn
-    
+
     def index_document(self, conn, d):
         if hasattr(self.bench, "process_document_xappy"):
             self.bench.process_document_xappy(d)
@@ -199,36 +214,38 @@ class XappyModule(Module):
 
     def finish(self, conn):
         conn.flush()
-        
+
     def searcher(self):
-        path = os.path.join(self.options.dir, "%s_xappy" % self.options.indexname)
+        path = os.path.join(self.options.dir, "%s_xappy"
+                            % self.options.indexname)
         return xappy.SearchConnection(path)
-        
+
     def query(self, conn):
         return conn.query_parse(" ".join(self.args))
-    
+
     def find(self, conn, q):
         return conn.search(q, 0, int(self.options.limit))
-    
+
     def findterms(self, conn, terms):
         limit = int(self.options.limit)
         for term in terms:
             q = conn.query_field(self.bench.spec.main_field, term)
             yield conn.search(q, 0, limit)
-    
+
     def results(self, r):
         hf = self.bench.spec.headline_field
         mf = self.bench.spec.main_field
         for hit in r:
             yield self._process_result({hf: hit.data[hf], mf: hit.data[mf]})
-        
+
 
 class XapianModule(Module):
     def indexer(self, **kwargs):
-        path = os.path.join(self.options.dir, "%s_xapian" % self.options.indexname)
+        path = os.path.join(self.options.dir, "%s_xapian"
+                            % self.options.indexname)
         self.database = xapian.WritableDatabase(path, xapian.DB_CREATE_OR_OPEN)
         self.ixer = xapian.TermGenerator()
-        
+
     def index_document(self, d):
         if hasattr(self.bench, "process_document_xapian"):
             self.bench.process_document_xapian(d)
@@ -238,31 +255,32 @@ class XapianModule(Module):
         self.ixer.set_document(doc)
         self.ixer.index_text(d[self.bench.spec.main_field])
         self.database.add_document(doc)
-        
+
     def finish(self, **kwargs):
         self.database.flush()
-        
+
     def searcher(self):
-        path = os.path.join(self.options.dir, "%s_xappy" % self.options.indexname)
+        path = os.path.join(self.options.dir, "%s_xappy"
+                            % self.options.indexname)
         self.db = xapian.Database(path)
         self.enq = xapian.Enquire(self.db)
         self.qp = xapian.QueryParser()
         self.qp.set_database(self.db)
-        
+
     def query(self):
         return self.qp.parse_query(" ".join(self.args))
-    
+
     def find(self, q):
         self.enq.set_query(q)
         return self.enq.get_mset(0, int(self.options.limit))
-    
+
     def findterms(self, terms):
         limit = int(self.options.limit)
         for term in terms:
             q = self.qp.parse_query(term)
             self.enq.set_query(q)
             yield self.enq.get_mset(0, limit)
-    
+
     def results(self, matches):
         hf = self.bench.spec.headline_field
         mf = self.bench.spec.main_field
@@ -277,58 +295,59 @@ class SolrModule(Module):
         self.conn = pysolr.Solr(self.options.url)
         self.conn.delete("*:*")
         self.conn.commit()
-    
+
     def index_document(self, d):
         self.solr_doclist.append(d)
         if len(self.solr_doclist) >= int(self.options.batch):
             self.conn.add(self.solr_doclist, commit=False)
             self.solr_doclist = []
-        
+
     def finish(self, **kwargs):
         if self.solr_doclist:
             self.conn.add(self.solr_doclist)
         del self.solr_doclist
         self.conn.optimize(block=True)
-        
+
     def searcher(self):
         self.solr = pysolr.Solr(self.options.url)
-    
+
     def query(self):
         return " ".join(self.args)
-    
+
     def find(self, q):
         return self.solr.search(q, limit=int(self.options.limit))
-    
+
     def findterms(self, terms):
         limit = int(self.options.limit)
         for term in terms:
             yield self.solr.search("body:" + term, limit=limit)
-    
+
 
 class ZcatalogModule(Module):
     def indexer(self, **kwargs):
-        from ZODB.FileStorage import FileStorage
-        from ZODB.DB import DB
-        from zcatalog import catalog
-        from zcatalog import indexes
-        import transaction
-        
-        dir = os.path.join(self.options.dir, "%s_zcatalog" % self.options.indexname)
+        from ZODB.FileStorage import FileStorage  #@UnresolvedImport
+        from ZODB.DB import DB  #@UnresolvedImport
+        from zcatalog import catalog  #@UnresolvedImport
+        from zcatalog import indexes  #@UnresolvedImport
+        import transaction  #@UnresolvedImport
+
+        dir = os.path.join(self.options.dir, "%s_zcatalog"
+                           % self.options.indexname)
         if os.path.exists(dir):
             rmtree(dir)
         os.mkdir(dir)
-        
+
         storage = FileStorage(os.path.join(dir, "index"))
         db = DB(storage)
         conn = db.open()
-        
+
         self.cat = catalog.Catalog()
         self.bench.spec.zcatalog_setup(self.cat)
         conn.root()["cat"] = self.cat
         transaction.commit()
-        
+
         self.zcatalog_count = 0
-    
+
     def index_document(self, d):
         if hasattr(self.bench, "process_document_zcatalog"):
             self.bench.process_document_zcatalog(d)
@@ -336,39 +355,40 @@ class ZcatalogModule(Module):
         self.cat.index_doc(doc)
         self.zcatalog_count += 1
         if self.zcatalog_count >= 100:
-            import transaction
+            import transaction  #@UnresolvedImport
             transaction.commit()
             self.zcatalog_count = 0
-        
+
     def finish(self, **kwargs):
-        import transaction
+        import transaction  #@UnresolvedImport
         transaction.commit()
         del self.zcatalog_count
-        
+
     def searcher(self):
-        from ZODB.FileStorage import FileStorage
-        from ZODB.DB import DB
-        from zcatalog import catalog
-        from zcatalog import indexes
-        import transaction
-        
-        path = os.path.join(self.options.dir, "%s_zcatalog" % self.options.indexname, "index")
+        from ZODB.FileStorage import FileStorage  #@UnresolvedImport
+        from ZODB.DB import DB  #@UnresolvedImport
+        from zcatalog import catalog  #@UnresolvedImport
+        from zcatalog import indexes  #@UnresolvedImport
+        import transaction  #@UnresolvedImport
+
+        path = os.path.join(self.options.dir, "%s_zcatalog"
+                            % self.options.indexname, "index")
         storage = FileStorage(path)
         db = DB(storage)
         conn = db.open()
-        
+
         self.cat = conn.root()["cat"]
-    
+
     def query(self):
         return " ".join(self.args)
-    
+
     def find(self, q):
         return self.cat.searchResults(body=q)
-    
+
     def findterms(self, terms):
         for term in terms:
             yield self.cat.searchResults(body=term)
-    
+
     def results(self, r):
         hf = self.bench.spec.headline_field
         mf = self.bench.spec.main_field
@@ -382,8 +402,9 @@ class NucularModule(Module):
     def indexer(self, create=True):
         import shutil
         from nucular import Nucular
-        
-        dir = os.path.join(self.options.dir, "%s_nucular" % self.options.indexname)
+
+        dir = os.path.join(self.options.dir, "%s_nucular"
+                           % self.options.indexname)
         if create:
             if os.path.exists(dir):
                 shutil.rmtree(dir)
@@ -392,52 +413,53 @@ class NucularModule(Module):
         if create:
             self.archive.create()
         self.count = 0
-    
+
     def index_document(self, d):
         try:
             self.archive.indexDictionary(str(self.count), d)
         except ValueError:
-            print "d=", d
+            print("d=", d)
             raise
         self.count += 1
         if not self.count % int(self.options.batch):
             t = now()
             self.archive.store(lazy=True)
             self.indexer(create=False)
-    
+
     def finish(self, **kwargs):
         self.archive.store(lazy=False)
         self.archive.aggregateRecent(fast=False, verbose=True)
         self.archive.moveTransientToBase(verbose=True)
         self.archive.cleanUp()
-    
+
     def searcher(self):
         from nucular import Nucular
-        
-        dir = os.path.join(self.options.dir, "%s_nucular" % self.options.indexname)
+
+        dir = os.path.join(self.options.dir, "%s_nucular"
+                           % self.options.indexname)
         self.archive = Nucular.Nucular(dir)
-    
+
     def query(self):
         return " ".join(self.args)
-    
+
     def find(self, q):
         return self.archive.dictionaries(q)
-        
+
     def findterms(self, terms):
         for term in terms:
             q = self.archive.Query()
             q.anyWord(term)
             yield q.resultDictionaries()
-            
-    
+
+
 class Bench(object):
     libs = {"whoosh": WhooshModule, "xappy": XappyModule,
             "xapian": XapianModule, "solr": SolrModule,
             "zcatalog": ZcatalogModule, "nucular": NucularModule}
-    
+
     def index(self, lib):
-        print "Indexing with %s..." % lib
-        
+        print("Indexing with %s..." % lib)
+
         options = self.options
         every = None if options.every is None else int(options.every)
         merge = options.merge
@@ -446,11 +468,11 @@ class Bench(object):
         upto = int(options.upto)
         count = 0
         skipc = skip
-        
+
         starttime = chunkstarttime = now()
-        
+
         lib.indexer()
-        
+
         for d in self.spec.documents():
             skipc -= 1
             if not skipc:
@@ -460,50 +482,50 @@ class Bench(object):
                 if chunk and not count % chunk:
                     t = now()
                     sofar = t - starttime
-                    print "Done %d docs, %0.3f secs for %d, %0.3f total, %0.3f docs/s" % (count, t - chunkstarttime, chunk, sofar, count / sofar)
+                    print("Done %d docs, %0.3f secs for %d, %0.3f total, %0.3f docs/s" % (count, t - chunkstarttime, chunk, sofar, count / sofar))
                     chunkstarttime = t
                 if count > upto:
                     break
                 if every and not count % every:
-                    print "----Commit"
+                    print("----Commit")
                     lib.finish(merge=merge)
                     lib.indexer(create=False)
-        
+
         spooltime = now()
-        print "Spool time:", spooltime - starttime
+        print("Spool time:", spooltime - starttime)
         lib.finish(merge=merge)
         committime = now()
-        print "Commit time:", committime - spooltime
+        print("Commit time:", committime - spooltime)
         totaltime = committime - starttime
-        print "Total time to index %d documents: %0.3f secs (%0.3f minutes)" % (count, totaltime, totaltime / 60.0)
-        print "Indexed %0.3f docs/s" % (count / totaltime)
-    
+        print("Total time to index %d documents: %0.3f secs (%0.3f minutes)" % (count, totaltime, totaltime / 60.0))
+        print("Indexed %0.3f docs/s" % (count / totaltime))
+
     def search(self, lib):
         lib.searcher()
-        
+
         t = now()
         q = lib.query()
-        print "Query:", q
+        print("Query:", q)
         r = lib.find(q)
-        print "Search time:", now() - t
-        
+        print("Search time:", now() - t)
+
         t = now()
         self.spec.print_results(lib.results(r))
-        print "Print time:", now() - t
-    
+        print("Print time:", now() - t)
+
     def search_file(self, lib):
         f = open(self.options.termfile, "rb")
         terms = [line.strip() for line in f]
         f.close()
-        
-        print "Searching %d terms with %s" % (len(terms), lib)
+
+        print("Searching %d terms with %s" % (len(terms), lib))
         lib.searcher()
         starttime = now()
         for r in lib.findterms(terms):
             pass
         searchtime = now() - starttime
-        print "Search time:", searchtime, "searches/s:", float(len(terms)) / searchtime
-    
+        print("Search time:", searchtime, "searches/s:", float(len(terms)) / searchtime)
+
     def _parser(self, name):
         p = OptionParser()
         p.add_option("-x", "--lib", dest="lib",
@@ -560,24 +582,24 @@ class Bench(object):
                      help="Store the body text in index", default=False)
         p.add_option("-q", "--snippets", dest="snippets", action="store_true",
                      help="Show highlighted snippets", default=False)
-        
+
         return p
-    
+
     def run(self, specclass):
         parser = self._parser(specclass.name)
         options, args = parser.parse_args()
         self.options = options
         self.args = args
-        
+
         if options.lib not in self.libs:
             raise Exception("Unknown library: %r" % options.lib)
         lib = self.libs[options.lib](self, options, args)
-        
+
         self.spec = specclass(options, args)
-        
+
         if options.setup:
             self.spec.setup()
-        
+
         action = self.search
         if options.index:
             action = self.index
@@ -585,17 +607,5 @@ class Bench(object):
             action = self.search_file
         if options.generate:
             action = self.generate_search_file
-        
+
         action(lib)
-        
-
-
-
-
-
-
-
-
-
-
-
